@@ -7,9 +7,6 @@ Different OAuth 2.0 grant types.
 
 ----
 """
-
-from typing import Tuple
-
 from .errors import (
     InvalidGrantError,
     InvalidRequestError,
@@ -21,18 +18,21 @@ from .models import Client
 from .requests import Request
 from .responses import TokenResponse
 from .storage import BaseStorage
-from .utils import decode_auth_headers, enforce_list, enforce_str, generate_token
+from .utils import enforce_list, enforce_str, generate_token
 
 
 class GrantTypeBase:
     """Base grant type that all other grant types inherit from."""
 
-    def __init__(self, storage: BaseStorage):
+    def __init__(self, storage: BaseStorage, client_id: str, client_secret: str):
         self.storage = storage
+        self.client_id = client_id
+        self.client_secret = client_secret
 
-    async def create_token_response(self, request: Request) -> TokenResponse:
+    async def create_token_response(
+        self, request: Request, client: Client
+    ) -> TokenResponse:
         """Creates token response to reply to client."""
-        client = await self.validate_request(request)
         token = await self.storage.create_token(
             request,
             client.client_id,
@@ -52,10 +52,8 @@ class GrantTypeBase:
 
     async def validate_request(self, request: Request) -> Client:
         """Validates the client request to ensure it is valid."""
-        client_id, client_secret = self.get_client_credentials(request)
-
         client = await self.storage.get_client(
-            request, client_id=client_id, client_secret=client_secret
+            request, client_id=self.client_id, client_secret=self.client_secret
         )
 
         if not client:
@@ -63,7 +61,7 @@ class GrantTypeBase:
                 request=request, description="Invalid client_id parameter value."
             )
 
-        if not client.check_grant_type(request.post.grant_type):  # type: ignore
+        if not client.check_grant_type(request.post.grant_type):
             raise UnauthorizedClientError(request=request)
 
         if not client.check_scope(request.post.scope):
@@ -71,15 +69,6 @@ class GrantTypeBase:
 
         return client
 
-    def get_client_credentials(self, request: Request) -> Tuple[str, str]:
-        client_id = request.post.client_id
-        client_secret = request.post.client_secret
-
-        if client_id is None or client_secret is None:
-            client_id, client_secret = decode_auth_headers(request)
-
-        return client_id, client_secret
-
 
 class AuthorizationCodeGrantType(GrantTypeBase):
     """
@@ -140,11 +129,20 @@ class AuthorizationCodeGrantType(GrantTypeBase):
         if authorization_code.is_expired:
             raise InvalidGrantError(request=request)
 
+        return client
+
+    async def create_token_response(
+        self, request: Request, client: Client
+    ) -> TokenResponse:
+        token_response = await super().create_token_response(request, client)
+
         await self.storage.delete_authorization_code(
-            request, client.client_id, request.post.code
+            request,
+            client.client_id,
+            request.post.code,  # type: ignore
         )
 
-        return client
+        return token_response
 
 
 class PasswordGrantType(GrantTypeBase):
@@ -185,10 +183,10 @@ class RefreshTokenGrantType(GrantTypeBase):
     See `RFC 6749 section 1.5 <https://tools.ietf.org/html/rfc6749#section-1.5>`_.
     """
 
-    async def create_token_response(self, request: Request) -> TokenResponse:
+    async def create_token_response(
+        self, request: Request, client: Client
+    ) -> TokenResponse:
         """Validate token request and create token response."""
-        client = await self.validate_request(request)
-
         old_token = await self.storage.get_token(
             request=request,
             client_id=client.client_id,
