@@ -6,21 +6,30 @@
 Storage helper class for storing and retrieving client and resource
 owner information. See the examples on the sidebar to view this in
 action.
+
 ----
 """
 
-from aioauth.types import TokenType
 import time
 from typing import Optional
 
+from aioauth.types import TokenType
+
 from .models import AuthorizationCode, Client, Token
 from .requests import Request
-from .utils import generate_token
 
 
 class BaseStorage:
-    async def create_token(self, request: Request, client_id: str, scope: str) -> Token:
+    async def create_token(
+        self,
+        request: Request,
+        client_id: str,
+        scope: str,
+        access_token: str,
+        refresh_token: str,
+    ) -> Token:
         """Generates a user token and stores it in the database.
+
         Warning:
             Generated token *must* be stored in the database.
         Note:
@@ -37,8 +46,8 @@ class BaseStorage:
             client_id=client_id,
             expires_in=request.settings.TOKEN_EXPIRES_IN,
             refresh_token_expires_in=request.settings.REFRESH_TOKEN_EXPIRES_IN,
-            access_token=generate_token(42),
-            refresh_token=generate_token(48),
+            access_token=access_token,
+            refresh_token=refresh_token,
             issued_at=int(time.time()),
             scope=scope,
             revoked=False,
@@ -54,6 +63,7 @@ class BaseStorage:
         refresh_token: Optional[str] = None,
     ) -> Optional[Token]:
         """Gets existing token from the database.
+
         Note:
             Method is used by
             :py:class:`aioauth.server.AuthorizationServer`,  and by the
@@ -77,8 +87,10 @@ class BaseStorage:
         redirect_uri: str,
         code_challenge_method: Optional[str],
         code_challenge: Optional[str],
+        code: str,
     ) -> AuthorizationCode:
         """Generates an authorization token and stores it in the database.
+
         Warning:
             Generated authorization token *must* be stored in the database.
         Note:
@@ -96,7 +108,7 @@ class BaseStorage:
             An :py:class:`aioauth.models.AuthorizationCode` object.
         """
         authorization_code = AuthorizationCode(
-            code=generate_token(48),
+            code=code,
             client_id=client_id,
             redirect_uri=redirect_uri,
             response_type=response_type,
@@ -117,17 +129,19 @@ class BaseStorage:
         redirect_uri: str,
         nonce: str,
     ) -> str:
-        """Returns token_id. Required for OpenID.
+        """Returns an id_token.
+        For more information see `OpenID Connect Core 1.0 incorporating errata set 1 section 2 <https://openid.net/specs/openid-connect-core-1_0.html#IDToken>`_.
 
-        Method is used by response types:
-            - ResponseTypeIdToken
+        Note:
+            Method is used by response type :py:class:`aioauth.response_type.ResponseTypeIdToken`
         """
-        raise NotImplementedError("create_token_id must be implemented.")
+        raise NotImplementedError("get_id_token must be implemented.")
 
     async def get_client(
         self, request: Request, client_id: str, client_secret: Optional[str] = None
     ) -> Optional[Client]:
         """Gets existing client from the database if it exists.
+
         Warning:
             If client does not exists in database this method *must*
             return ``None`` to indicate to the validator that the
@@ -146,6 +160,7 @@ class BaseStorage:
 
     async def authenticate(self, request: Request) -> bool:
         """Authenticates a user.
+
         Note:
             This method is used by the grant type
             :py:class:`aioauth.grant_type.PasswordGrantType`.
@@ -161,6 +176,7 @@ class BaseStorage:
         self, request: Request, client_id: str, code: str
     ) -> Optional[AuthorizationCode]:
         """Gets existing authorization code from the database if it exists.
+
         Warning:
             If authorization code does not exists this function *must*
             return ``None`` to indicate to the validator that the
@@ -183,11 +199,12 @@ class BaseStorage:
         self, request: Request, client_id: str, code: str
     ) -> None:
         """Deletes authorization code from database.
+
         Note:
             This method is used by the grant type
             :py:class:`aioauth.grant_type.AuthorizationCodeGrantType`.
         Args:
-            An :py:class:`aioauth.requests.Request`.
+            request: An :py:class:`aioauth.requests.Request`.
             client_id: A user client ID.
             code: An authorization code.
         """
@@ -197,6 +214,7 @@ class BaseStorage:
 
     async def revoke_token(self, request: Request, refresh_token: str) -> None:
         """Revokes a token's from the database.
+
         Note:
             This method *must* set ``revoked`` to ``True`` for an
             existing token record. This method is used by the grant type
